/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef CDAQ_H
#define CDAQ_H

#include <vector>
#include <string>

#include "XML_enum.h"
#include "Stream_enum.h"
#include "DAQ_enum.h"
#include "DAQError.h"

#include "DAQ_struct.h"

#include "CXml.h"
#include "CReader.h"
#include "CFormatter.h"

using namespace std;


 /** \file CDaq.h 
 *\date  Created on: 31-mag-2009
 *\author  Author: Claudio Salvadori, Christian Nastasi and Paolo Pagano
 *\brief User interface from the data acquisition library and the user application*/
 
  /** \class CDaq
   *\brief User interface from the data acquisition library and the user application*/
class CDaq{
	public:

		/**\brief CDaq constructor
		* \param xmlFile	pointer to the string of the xml file path*/
		CDaq(char *xmlFile);
		
		/**\brief CDaq destructor*/
		~CDaq();
		
		/**\brief Init method to initialize the background classes with the data parsed
			from the xml file*/
		daq_ret init(void);
		
		/**\brief Method that returns the title of the title
		* \return Returns the string of the title */
		string getTitle(void);
		
 		/**\brief Method that formats data and create the vector of data formatted
		*\param buffer 	Pointer to a vector that contain the n formatted data vector 
		*		related to the n messages received from the reader(output)
		* \return Returns the string of the title */
		daq_ret getData(vector< vector<SData> >* data);
		
	
	protected:

		IXml					*daq_Xml;		/**< Instance of the xml parser interface. */
		IStreamReader				*daq_Reader;		/**< Instance of data reader interface. */
		IFormatter				*daq_Formatter;		/**< Instance of the formatter interface */

		
	private:
	
		string				daq_title;		/**< Variable that contains the title */
		struct SDef			*daq_Definition;	/**< Varibale of SDef struct that define the stream \see SDef */
		vector<struct SMData>		*daq_Metadata;		/**< Vector of metadata \see SMData */
		vector<struct SMessage>		*daq_Message;		/**< Vector of the messages (output of the reader) */
		vector<SData>			*daq_Data;		/**< Formatted data vector \see SData*/
		
};

#endif

