/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CHISTOGRAM_H
#define CHISTOGRAM_H

#include <stdio.h>
#include <string>

#include "TROOT.h"
#include "TCanvas.h"
#include "TCanvas.h"

#include "TH1C.h"
#include "TH1S.h"
#include "TH1I.h"
#include "TH1F.h"
#include "TH1D.h"

#include "IRenderer.h"
#include "CXmlParser.h"

 /**
 *\file CHistogram.h
 *
 *\date  Created on: 01-oct-2009
 *\author  Author: Claudio Salvadori
 *\brief  Class to render a an histogram using the root libraries
 */
 
 /**
 *\class CHistogram
 *\brief  Implementation of IRenderer interface to render an 
 * 	histogram using root libraries
 */
 
 /**\todo Add possible fitting and error handler*/
 /**\todo Add some style stuff*/
class CHistogram: public IRenderer{
public:
	~CHistogram(void);
	
	/** 	\brief This function, used by the object factory, return an istance
	*		of CHistogram class
	*	\param node	pointer to a generic node (in this case DOMNode) where
	*			extracts the histogram presets.
	*/
	static IRenderer *createObj(const void* node);
	
	/** 	\brief Function to set the data type
	* 	\param vType	data type*/
	void setType(var_type vType){vType_ = vType;};
	
	/** 	\brief Function to render the histogram
	* 	\param data 	pointer to the vector/scalar to be mapped on histogram*/
	void render(void * data, unsigned long size);
private:
	/**	\brief The constructor function parses the xml node relative to histogram 
	* 	and canvas and intializes those.
	* 	\param node	pointer to parent DOM node*/
	CHistogram(const DOMNode* node);
		
	/**	\brief This function parse the xml-file to extract the histogram presets
	* 	\param node	pointer to parent DOM node*/
	void histoPreset(const DOMNode* node);
	
	/**	\brief This function parse the xml-file to extract the canvas presets
	* 	\param node	pointer to parent DOM node*/
	void canvasPreset(const DOMNode* node);
	
	/**	\brief Construct the root canvas*/
	void setCanvas(void);	
			
	/** 	\brief Function to fill the histogram
	* 	\param data 	pointer to the vector/scalar to be mapped on histogram*/
	/**\todo Insert an entire buffer like image histogram*/
	void fillHisto(double data);
	
	DOMNode* 		hnode_;		/**<Histogram presets DOM node*/
	DOMNode* 		cnode_;		/**<Canvas presets DOM node*/
	
	std::string		name_;		/**<Histogram name*/
	std::string		title_;		/**<Histogram title*/
	
	unsigned long 		nBins_;		/**<Number of bins*/
	double  		min_;		/**<x min value*/
	double	 		max_;		/**<x max value*/
	 
	std::string		cTitle_;	/**<Canvas title*/
	unsigned int		cWidth_;	/**<Canvas width*/
	unsigned int		cHeight_;	/**<Canvas height*/
	
	TH1D* 			histo_;		/**<Pointer to a ROOT histogram (Double type)*/
	TCanvas*		canv_;		/**<Pointer to a ROOT canvas*/
	
	var_type 		vType_;		/**<Histogram variable type*/
	
	unsigned int		counter;	/**<Counter for time*/
	unsigned int		version;	/**<Graph  version*/
	
	CXMLParser*		 parser_;	/**<Pointer to Xml parser*/
};

#endif
