/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CTABLE_H
#define CTABLE_H

#include <stdio.h>
#include <string>
#include <iostream>
#include <fstream>

#include "IRenderer.h"
#include "CXmlParser.h"

 /**
 *\file CTable.h
 *
 *\date  Created on: 01-oct-2009
 *\author  Author: Claudio Salvadori
 *\brief  Class to write a table inside a file
 */
 
 /**
 *\class CTable
 *\brief  Implementation of IRenderer interface to 
 * write a table inside a file
 */
 
class CTable:public IRenderer{
public:
	~CTable(void);
	
	/** 	\brief This function, used by the object factory, return an istance
	*		of CTable class
	*	\param node	pointer to a generic node (in this case DOMNode) where
	*			extracts the table presets.
	*/
	static IRenderer *createObj(const void* node);
	
	/** 	\brief Function to render the histogram
	* 	\param data 	pointer to the vector/scalar to be mapped on histogram*/
	void render(void * data, unsigned long size);
	
	/** 	\brief Function to set the data type
	* 	\param data 	pointer to the vector/scalar to be mapped on table*/
	void setType(var_type vType){vType_ = vType;};
private:
	/**	\brief Constructor: it intializes the file handler and the column counter
	* 	\param node	pointer to parent DOM node*/
	CTable(const DOMNode* node);
	
	void tablePreset(const DOMNode* node);
	
	DOMNode* 			tnode_;
	
	unsigned int			column_;
	std::string			name_;
	std::string			title_;
	var_type			vType_;
	
	std::ofstream 			file_;
	//static std::vector<std::string>	*nameV_;
	//static unsigned int	counter_;
	
	CXMLParser*		 	parser_;	/**<Pointer to Xml parser*/	
};

#endif
