/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file IMessageDaq.h
 *
 *\date  Created on: 25-apr-2009
 *\author  Author: Claudio Salvadori
 *\brief Interface for the messagger classes. Some application
 *	couldn't send single messages, but group of messages (for 
 *	example tables): the aim of the classes those implement
 *	this inteface is to devide the data received in single messages.
*	 This class is used by the CReader class.
 */
#ifndef IMESSAGEDAQ_H
#define IMESSAGEDAQ_H

#include <vector>
#include <string>

#include "Stream_enum.h"
#include "DAQ_struct.h"

/**\class IMessageDaq
 *\brief Interface for the messagger classes. Some application
 *	couldn't send single messages, but group of messages (for 
 *	example tables): the aim of the classes those implement
 *	this inteface is to devide the data received in single messages.
*	 This class is used by the CReader class.
 */
class IMessageDaq{
	public:
		virtual ~IMessageDaq() {};

		/**\brief Initialize the messages maker
		*\param def 	Pointer to a stream definition structure (input)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret getDefinition (struct SDef def) = 0;
		
		/**\brief This method divides the packet recived into message to be formatted (the lenght of each message is
		*		the pkt_lenght defined in the xml file)
		*\param buffer_i	Pointer of the buffet to decode (input)
		*\param length_i	Length of the buffer to decode (input)
		*\param buffer_o 	Pointer to a vector of the messages to be formatted (output)
		*\return This method return one of the possible value from s_ret \see s_ret*/
		virtual s_ret retStream(unsigned char *buffer_i, unsigned long length_i, std::vector<struct SMessage> *buffer_o) = 0;
		
};
#endif
