/*
Copyright (C) 2009 -  Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CSocket.cpp
 *\brief  Socket access functions develpoment
 */
#include "CSocket.h"

#define MAX_ACK_LENGHT 100

using namespace std;

CSocket::~CSocket(){
	Socket_close();
}

s_ret CSocket::Socket_handshaking(handshake_style style){
	s_ret help;
	unsigned char data[MAX_ACK_LENGHT];
//#ifdef DEBUG
	cout << "Handshake" << endl;
//#endif
	//handshaking if it exists...
	switch(style){
		case H_S_STANDARD:
		help = Socket_write((unsigned char*)(*so_Handshk)[0].h_Pkt, (*so_Handshk)[0].h_Size_P);
		if(help != S_RET_OK)
			return help;
		break;
		
		case H_S_NONE:
		return S_RET_OK;
		break;
		
		case H_S_START:
#ifdef DEBUG
			cout << "Number of handshking packet: " << so_Handshk->size() << endl;
#endif
		for(unsigned int i = 0; i < so_Handshk->size(); i++){
			//send handshking message
#ifdef DEBUG
			cout << "Writing: " << (*so_Handshk)[i].h_Pkt << endl;
#endif
			help = Socket_write((unsigned char*)(*so_Handshk)[i].h_Pkt, (*so_Handshk)[i].h_Size_P);
			if(help != S_RET_OK)
				return help;

			if(((*so_Handshk)[i].h_Size_A != 0) && ((*so_Handshk)[i].h_Size_N != 0)){
							
				//reading answer
				help = Socket_read(data, (*so_Handshk)[i].h_Size_A);
				if(help != S_RET_OK)
					return help;

				cout << "Reading: " << data << endl;

				if(memcmp((*so_Handshk)[i].h_Ack, data, (*so_Handshk)[i].h_Size_A) != 0){
					if(memcmp((*so_Handshk)[i].h_Nack, data, (*so_Handshk)[i].h_Size_N) == 0)
						return S_SO_NACK;
					else 
						return S_SO_GEN_ERR;
				}
			}
		}
		break;
	}
	
	// shutdown the connection for sending since no more data will be sent
	// the client can still use the ConnectSocket for receiving data
	#if defined(__WIN32__)
	if(shutdown(so_handle, SD_SEND) == SOCKET_ERROR){
		Socket_close();
		return S_SO_GEN_ERR;
	}
	
	#else

	#endif
	return S_RET_OK;
}
	
s_ret CSocket::getDefinition (struct SDef def, s_read_style sty){	
	so_Style = sty;
	
	so_Port = def.s_Sock_Port;
	so_Ip_Addr = (char*)def.s_Name.c_str();

	switch(so_Style){
		case S_STYLE_LENGTH:
		so_Size = def.s_Size;
		#ifdef DEBUG
		printf("Lenght style\n");
		#endif
		break;
	
		case S_STYLE_S_E:
		so_Size = def.s_Size;
		so_Start = (unsigned char)def.s_Option->o_Start;
		so_End = (unsigned char)def.s_Option->o_End;
		#ifdef DEBUG
		printf("Start/end style \n");
		#endif
		break;
	}
	
	so_HStyle = def.s_Style;
	
	so_Handshk = def.s_Handshk;

	if(so_HStyle == H_S_START){
		s_ret help;
		help = Socket_init();
		if(help != S_RET_OK)
			return help;
		
		help = Socket_handshaking(so_HStyle);
		
		//Handshake did! disable all the eventual future possibilities to have an hanshake!
		so_HStyle = H_S_NONE;
		
		return help;
	}else{
		return Socket_init();
	}
	
	return S_RET_OK;
}

//Blocking function
s_ret CSocket::retStream(unsigned char **buffer, unsigned long *length){
	s_ret help;
	switch(so_Style){		
		case S_STYLE_LENGTH:
		if(so_HStyle == H_S_STANDARD){
			help = Socket_handshaking(so_HStyle);
			if(help != S_RET_OK)
				return help;
		}
		
		*buffer = new unsigned char[so_Size];
		help = Socket_read(*buffer, (unsigned int) so_Size);
		*length = so_Size_r;
		
		#ifdef DEBUG
		for(unsigned long i = 0; i < *length; i++)
			printf("Character received: %x, %d\n", (int)(*buffer)[i], (int)(*buffer)[i]);
		#endif
		
		return help;
		break;
		
		case S_STYLE_S_E:
		std::vector<unsigned char> buff;
		unsigned char buff_t;
		unsigned int size = 1;
		
		if(so_HStyle == H_S_STANDARD){
			help = Socket_handshaking(so_HStyle);
			if(help != S_RET_OK)
				return help;
		}
		
		//Searching for syncro character (start character)...
		while(buff_t != so_Start){				
			help = Socket_read(&buff_t, size);
			if(help != S_RET_OK)
				return help;
			#ifdef DEBUG
			printf("%d", buff_t);
			#endif
		}
		
		//clear the buffer
		buff.clear();

		//... and reading untill end character
		while(1){
			help = Socket_read(&buff_t, size);
			if(help != S_RET_OK)
				return help;
				
			#ifdef DEBUG
			printf("%d", buff_t);
			#endif
			
			//End character found	
			if((buff_t == so_End) && (buff.size() >= so_Size)){
				*length = buff.size();
				*buffer = new unsigned char[*length];
				
				for(unsigned long i_s = 0; i_s < *length; i_s++) 
					(*buffer)[i_s] = buff[i_s];
				
				#ifdef DEBUG
				for(unsigned long i_s = 0; i_s < *length; i_s++){
					//printf("Character received: %x, %d\n", (int)(*buffer)[i_s], (int)(*buffer)[i_s]);
					printf("Character received: %x, %d\n", (int)buff[i_s], (int)buff[i_s]);
				}
				#endif
				return S_RET_OK;
			}
	
			//push back the character
			buff.push_back(buff_t);
		};
		break;
	}
	
	return S_RET_OK;
}

s_ret CSocket::Socket_init(void){
	#if defined(__WIN32__) /* WIN32 Implementation */
	WORD version;
	int error;
	
	/* WinSock initializing */
	version = MAKEWORD( 2, 0 );

	error = WSAStartup( version, &so_wsaData );

	/* check for error */
	if ( error != 0 )
		/* error occured */
		return S_SO_INIT_ERR;

	/* check for correct version */
	if ( LOBYTE( so_wsaData.wVersion ) != 2 || HIBYTE( so_wsaData.wVersion ) != 0 ){
		/* incorrect WinSock version */
		WSACleanup();
		return S_SO_INIT_V_ERR;
	}
	
	struct addrinfo *result = NULL;
	struct addrinfo *ptr = NULL;
	struct addrinfo hints;
	
	//Setting up the hints
	ZeroMemory(&hints, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;

	// Resolve the server address and port
	if(getaddrinfo(so_Ip_Addr, so_Port, &hints, &result) != 0){
	    WSACleanup();
	    return S_SO_INIT_R_ERR;
	}
	
	//Initialize socket value
	so_handle = INVALID_SOCKET;
	
	//Socket creation (client side)
	for(ptr=result; ptr != NULL; ptr=ptr->ai_next){
		so_handle = socket(ptr->ai_family, ptr->ai_socktype, ptr->ai_protocol);
		if(so_handle == SOCKET_ERROR){
			WSACleanup();
			return S_SO_INIT_C_ERR;
		}

		// Connect to server.
		if (connect(so_handle, ptr->ai_addr, (int)ptr->ai_addrlen) == SOCKET_ERROR){
		    Socket_close();
		    so_handle = INVALID_SOCKET;
			return S_SO_INIT_CONN_ERR;
		}

		freeaddrinfo(result);
	}
	
	return S_RET_OK;
	#else 									/* Unix-like Implementation */
	
	// Create a socket
	so_handle = socket(AF_INET, SOCK_STREAM, 0);
	if (so_handle < 0)	
		return S_SO_INIT_C_ERR;

	//Look up server to get numeric IP address
	struct hostent * record = gethostbyname(so_Ip_Addr);
	if (record==NULL) 
		return S_SO_INIT_R_ERR;
		
	in_addr * addressptr = (in_addr *) record->h_addr;
	
	// Create a sockaddr_in to describe the server
	struct sockaddr_in server_info;
	bzero((char *) &server_info, sizeof(server_info));

	//Fill sockaddr_in structure
	server_info.sin_family = AF_INET;
	bcopy((char *)record->h_addr, 
		(char *)&server_info.sin_addr.s_addr,
		record->h_length);

	server_info.sin_port = htons(so_Port);

	// Connect
	if (connect(so_handle, (sockaddr *) &server_info, sizeof(server_info)) < 0)
		return S_SO_INIT_CONN_ERR;
	
	//#ifdef DEBUG
	printf("Connected to %s\n", inet_ntoa(* addressptr));
	//#endif
	
	return S_RET_OK;
	#endif
}

void CSocket::Socket_close(void){
#if defined(__WIN32__) 	/* WIN32 Implementation */
	//Closing Socket
	closesocket(so_handle);
	//WinSock: Shutdown
	WSACleanup();
#else			/* Unix-like Implementation */
	close(so_handle);
#endif
}

s_ret CSocket::Socket_read(unsigned char *data, unsigned int data_size){
	int read_size;
	read_size = read(so_handle, (char*)data, (int)data_size);

#ifdef DEBUG
	cout << "Data: " << endl;
	for(unsigned int i = 0; i < read_size; i ++)
		cout << hex << (int)*(data + i) << ", " << endl;
#endif
	
	if (read_size > 0){
		so_Size_r = (unsigned int)read_size;
		if((unsigned int)read_size == data_size){
			return S_RET_OK;
		}else{
			return S_SO_LENGHT_ERR;
		}
	}else if (read_size == 0){
		return S_SO_CLOSED_ERR;
	}else{
		return S_SO_READ_ERR;
	}

	return S_RET_OK;
}

s_ret CSocket::Socket_write(unsigned char* data, unsigned int data_size){
	// Send an initial buffer
	if(write(so_handle, (char*)data, (int)data_size) < data_size){
		Socket_close();
		return S_SO_WRITE_ERR;
	}
	return S_RET_OK;

}

